﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;

namespace TestAddingRowsToGrid
{
    public partial class Form1 : Form
    {
        string TEXT = "Text";
        public Form1()
        {
            InitializeComponent();
        }

        private void gridRows_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            e.Layout.Override.CellMultiLine = Infragistics.Win.DefaultableBoolean.True;
            e.Layout.Override.RowSizing = RowSizing.AutoFree;
            for (int i = 1; i < e.Layout.Bands.Count; i++)
            {
                e.Layout.Bands[i].ColHeadersVisible = false;
                e.Layout.Bands[i].Columns[TEXT].CellClickAction = CellClickAction.Edit;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            gridRows.Rows[0].ExpandAll();
        }

        private void gridRows_BeforeEnterEditMode(object sender, CancelEventArgs e)
        {
            if (gridRows.ActiveRow.Band.Index != 0)
            {
                using (TextBox tb = new TextBox())
                {
                    string activeBandKey = gridRows.ActiveRow.Band.Key;
                    StringBuilder textStringBuilder = new StringBuilder();
                    for (int rowInt = 0; rowInt < gridRows.Rows[0].ChildBands[activeBandKey].Rows.Count; rowInt++)
                    {
                        textStringBuilder.Append(gridRows.Rows[0].ChildBands[activeBandKey].Rows[rowInt].Cells[TEXT].Value.ToString());
                    }
                    tb.Size = new Size(gridRows.ActiveCell.Width, gridRows.ActiveCell.Height);
                    tb.Multiline = true;
                    frmTextbox textbox = new frmTextbox(textStringBuilder.ToString());
                    textbox.ShowDialog(this);
                    using (Graphics graphics = tb.CreateGraphics())
                    {
                        int maxRowHeight = (gridRows.Font.Height * 5) + 2;
                        string[] arrayOfAnswer = SplitStringBySpacesAndLineFeeds(textbox.Answer);
                        List<string> listOfAnswers = new List<string>();
                        StringBuilder sb = new StringBuilder();
                        foreach (string answerSection in arrayOfAnswer)
                        {
                            if (!string.IsNullOrEmpty(answerSection))
                            {
                                sb.Append(answerSection);
                                if (graphics.MeasureString(sb.ToString(), gridRows.Font, tb.Width).Height > maxRowHeight)
                                {
                                    listOfAnswers.Add(sb.ToString().Remove(sb.ToString().LastIndexOf(answerSection)));
                                    sb = new StringBuilder();
                                    sb.Append(answerSection + " ");
                                }
                            }
                        }
                        listOfAnswers.Add(sb.ToString());
                        gridRows.ActiveCell.Value = listOfAnswers[0];
                        gridRows.ActiveRow.PerformAutoSize();
                        int rowCount = gridRows.Rows[0].ChildBands[activeBandKey].Rows.Count;
                        for (int rowInt = 1; rowInt < rowCount; rowInt++)
                        {
                            gridRows.Rows[0].ChildBands[activeBandKey].Rows[rowInt].Delete(false);
                        }
                        gridRows.Rows[0].ChildBands[activeBandKey].Rows[0].Activate();
                        //now, values aren't being shown on the second update
                        for (int i = 1; i < listOfAnswers.Count; i++)
                        {
                            UltraGridRow row = gridRows.DisplayLayout.Bands[activeBandKey].AddNew();
                            row.Cells[TEXT].Value = listOfAnswers[i];
                            row.PerformAutoSize();
                        }
                        gridRows.ActiveCell = null;
                        gridRows.ActiveRow = null;
                    }
                }
            }
        }

        /// <summary>
        /// Splits the string by spaces and line feeds.
        /// </summary>
        /// <param name="stringToSplit">The string to split.</param>
        /// <returns></returns>
        private string[] SplitStringBySpacesAndLineFeeds(string stringToSplit)
        {
            string[] carriageReturn = new string[] { "\r\n" };
            int countSplitByCarriageReturn = stringToSplit.Split(new string[] { "\r\n" }, StringSplitOptions.None).Count();
            string[] splitBySpaces = stringToSplit.Split(' ');
            string[] splitString = new string[countSplitByCarriageReturn + splitBySpaces.Count()];
            int carriageReturnCount = 0;
            try
            {
                for (int i = 0; i < splitBySpaces.Count(); i++)
                {
                    string[] carriageReturnSplit = splitBySpaces[i].Split(carriageReturn, StringSplitOptions.None);
                    if (carriageReturnSplit.Count() > 1)
                    {
                        //do a for loop here, and if it is the last one, don't add a carriage return, add a space
                        for (int innerI = 0; innerI < carriageReturnSplit.Count(); innerI++)
                        {
                            //if this is the last one, need to add a space, not a carriage return
                            if (innerI == (carriageReturnSplit.Count() - 1))
                            {
                                splitString[i + carriageReturnCount] = carriageReturnSplit[innerI].Insert(carriageReturnSplit[innerI].Length, " ");
                            }
                            else
                            {
                                splitString[i + carriageReturnCount] = carriageReturnSplit[innerI].Insert(carriageReturnSplit[innerI].Length, "\r\n");
                                carriageReturnCount++;
                            }
                        }
                    }
                    else
                    {
                        if (i == (splitBySpaces.Count() - 1))
                        {
                            splitString[i + carriageReturnCount] = splitBySpaces[i];
                        }
                        else
                        {
                            splitString[i + carriageReturnCount] = splitBySpaces[i].Insert(splitBySpaces[i].Length, " ");
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                //ErrorLogger.LogError(ex, Application.ProductName, true);
            }
            return splitString;
        }
    }
}
